/* Node.js program */
import * as fs from 'fs';
import * as process from 'process';

import axios from "axios";
// https://developer.chrome.com/docs/puppeteer/
// Download problem: 'sudo npm i puppeteer@latest --unsafe-perm=true --allow-root'
// Check whether 'puppeteer.js' has been installed: 'npm list -depth=0'
import * as puppeteer from "puppeteer";

// brew install --cask chromium
// which chromium: '/usr/local/bin/chromium'

process.argv.forEach((value, index) => {
    console.log(`${index}: ${value}`);
});

class Carrefour_fr {

    static Path() {
        return '/s?q=';
    }

    static URL() {
        return 'https://www.carrefour.fr';
    }

    static async Take_picture() { // 'async' returns an 'AsyncFunction' object
        /* Key rule:
         * 'await' returns the promise result, say what is computed by 'puppeteer.launch'
         * *NOT USING* 'await' leads to get the promise itself!
         */
//    const call_without_await = puppeteer.launch(); // Better is possible...
//    console.log(call_without_await instanceof Promise); // 'true' is displayed!
        const browser = await puppeteer.launch({
            executablePath: '/usr/local/bin/chromium', // Mac!
            headless: false
        }); // Launch 'Chrome' *with* visual environment
        // Following call *DOES NOT* occur (because of prior 'await') until prior one terminates:
        const page = await browser.newPage();
        // 'https://www.carrefour.fr/s?q=' seule donne le nombre total d'items !
        await page.goto(Carrefour_fr.URL() + Carrefour_fr.Path() + (typeof process.argv[2] === 'string' ? process.argv[2] : '')); // By default, 'page.goto(...)' waits for the page load event
        const photo_file_name = (typeof process.argv[2] === 'string' ? process.argv[2] : 'undefined') + '.png';

        const body = await page.evaluate(() => document.body.innerHTML);
        /** Image URLs are at a given location: */
        const start = /["']images["']\s?:\s?\[\s?{/;
        const end = /]\s?}/;
        const image_URLs_location = body.substring(body.search(start), body.search(end));
//    console.log(image_URLs_location + '\n\n');

        // Raw sought string: '/media\/1500x1500\/Photosite\/PGC\/EPICERIE\/3560070478781_PHOTOSITE_20171208_164121_0.jpg?placeholder='
        // https://javascript.info/regular-expressions
//    const pattern = /\/media\\\/\d+x\d+\\\/(\b\w+\b\\\/)+/;
        // http://www.javascripter.net/faq/backslashinregularexpressions.htm
        const pattern = new RegExp('\/media\\\\/\\d+x\\d+\\\\/(\\b\\w+\\b\\\\/)+' + process.argv[2] + /* Checking enforcement -> */ '[\\w+]'); // Replace 'media' by '\\b\\w+\\b'?
//    console.log("Found? " + pattern.test(image_URLs_location)); // 'true'?
        const start_index = image_URLs_location.search(pattern);
        if (start_index !== -1) {
            const end_index = image_URLs_location.search(/\?placeholder=/);
            // First sought occurrence for simplification:
            const image_URL = Carrefour_fr.URL() + image_URLs_location.substring(start_index, end_index).replace(/\\/g, '');
//        console.log(image_URL);
            const blob: Blob = await axios.get(image_URL);
            fs.createWriteStream(photo_file_name).write(blob);
        } else
            await page.screenshot({path: photo_file_name}); // Default picture...
        await browser.close();
        return photo_file_name;
    }
}

Carrefour_fr.Take_picture().then((photo_file_name) => { // Instead of 'take_picture();', which is the "common" style of calling a function...
    console.log("Photo file name: " + photo_file_name);
});
